	;
	; ASM Software Monitor-52
	; Version 0.2
	;
	; (c) 2006 by Matthias Arndt / ASM Software <marndt@asmsoftware.de>
	;
	; THE GNU GENERAL PUBLIC LICENCE APPLIES TO THIS SOFTWARE!
	;
	; a monitor/debugger for use with the PHYtec microMODUL-8051
	;
	
VERSIONMAJOR	EQU 	00h
VERSIONMINOR	EQU	02h
	
	; for FLASH/ROM use..adjust this to A000h
monstart	CODE	6000h
	; address of user program
program		EQU	0100h
	; reset vector and RAMbased Interrupt table
resetvector	CODE	0000h

taskingcompat	EQU	918eh
jumptableoffset EQU	030h

	; stack location of the monitor
stackloc	DATA	090h

	; 8052 registers
T2CON		DATA	0C8h
T2RC2L		DATA	0CAh
T2RC2H		DATA	0CBh	
	
	; save space for user program registers
	; (e.q. amon52 will set this DPTR; PC, etc before executing the user program)
	; we use register bank 2 for this purpose!
userDPH		DATA 	010h
userDPL		DATA	011h
userPCH		DATA	012h
userPCL		DATA	013h
userACC		DATA	014h
userB		DATA	015h
userPSW		DATA	016h
userSP		DATA	017h	
	

	; baudrate setting
	; FFD9h = 9600 Baud (12Mhz)
	; FFF6h	= 38400 Baud (12 Mhz)
baudrate	EQU	0FFD9h
	
PROMPTCHAR	EQU	'>'

	; some ASCII characters
ESC		EQU	27
SPACE		EQU	' '
BACKSPACE	EQU 	8
TAB		EQU	9	
CR		EQU	13
LF		EQU	10
NUL		EQU	0
CTRLC		EQU	3h
	


	; some MACROs
REGS_USER 	MACRO
	clr	RS0
	clr	RS1
ENDM

REGS_MON	MACRO
	setb    RS0
        setb    RS1
ENDM

PUT_SPACE	MACRO
	mov A,#SPACE
	acall putchar
ENDM

PUT_SPACE2	MACRO
	mov A,#SPACE
	acall putchar
	acall putchar
ENDM

	; we use lcall putstring here because the SFR tables add a 2K gap into our code
CRLF		MACRO
	mov dptr,#crlfstr
	lcall putstring
ENDM

DO_BACKSPACE	MACRO
	mov A,#BACKSPACE
	acall putchar
	mov A,#' '
	acall putchar
	mov A,#BACKSPACE
	acall putchar
ENDM
	; for FLASH use
ORG resetvector
	ljmp poweron
	
	
	; Tasking compatibility
;ORG taskingcompat
	;ljmp warmstart
	
	; Interrupt table template (will be copied into XRAM)
ORG monstart
	; ljmp warmstart
	ljmp poweron
ORG monstart+3       ;ext int #0
        ljmp    program+3
ORG monstart+11      ;timer #0
        ljmp    program+11
ORG monstart+13h     ;external interrupt routine #1
        ljmp    program+13h     
ORG monstart+1bh     ;timer #1
        ljmp    program+1bh   
ORG monstart+23h     ;serial port
        ljmp    program+23h   
ORG monstart+2bh     ;timer #2 (8052 only)
        ljmp    program+2bh
	
	; jump table for open OS calls
ORG monstart+jumptableoffset
	ajmp monstart
	ajmp putchar
	ajmp putstring
	ajmp getchar
	ajmp putbyte
	ajmp getbyte
	
	; getchar - read a single character from the SIO
	; I: none
	; O: ACC - the char that was read
getchar:	
	
	jnb     RI,$  ; wait for character
        clr     RI
        mov     A,SBUF
        ret
	
	; putchar - outputs a single character to the SIO
	; I: ACC - char to output
	; O: none
putchar:	
	
        jnb     ti,$       ;wait if a character is still sending
        mov     SBUF,A
        clr     TI         ;note: hardware will set ti when the
        ret                ;      character finishes sending...
		
	; putstring - prints 0-terminated string from CODE space to the SIO
	; uses r1 for indirect addressing exclusively - use r0 elsewhere!
	; I: DPTR - address of string
	; O: none
putstring:
	push ACC
	mov r1,#0
psloop:
	mov A,r1
	movc A,@A+DPTR			; read char from code space
	jz psloopend			; terminate on NUL char
	acall putchar
	inc r1				; next char	
	sjmp psloop			; loop again
	
psloopend:
	pop ACC
	ret
	
	; putbyte: writes a byte as hex on the SIO
	; I: ACC - byte to send
	; O: none
putbyte:
	swap a
	acall putHalbbyte
	swap a
	acall putHalbbyte
	ret
putHalbbyte:
; E: acc.3 ... acc.0 (Halbbyte)
	push acc
	push psw
	acall nyb2asc
	acall putchar
	pop psw
	pop acc
	ret
nyb2asc:	
	anl	a, #0fh
	inc	a
	movc	a, @a+pc
	ret
   	DB	"0123456789ABCDEF"
	; getbyte: reads a byte in hex notation from the SIO	
	; I: none
	; O: ACC - the byte as read
getbyte:
	acall getchar			; read first char
	mov r4,A
	cjne A,#CR,gb1_noCR
	mov A,#0			; return 0 for whole routine if CR pressed here
	ret
gb1_noCR:
	acall validhex
	jc getbyte			; no valid hex char, so try again
	mov A,r4
	acall putchar			; show char
	mov r6,b			; save value
	
gb2:
	acall getchar			; read second char
	mov r4,a
	cjne A,#BACKSPACE,gb_noBS	; BACKSPACE?
	DO_BACKSPACE
	sjmp getbyte			; go back and read first again
gb_noBS:
	cjne A,#CR,gb2_noCR		; ENTER?
	mov A,r6
	mov r7,A
	mov r6,#0
	sjmp gb_finish
gb2_noCR:
	acall validhex
	jc gb2
	mov A,r4
	acall putchar			; show 2nd char
	mov r7,b
	
final_cr:
	acall getchar
	cjne A,#BACKSPACE,final_cr_noBS	; BACKSPACE?
	DO_BACKSPACE
	sjmp gb2			; read another nybble 2
final_cr_noBS:
	cjne A,#CR,final_cr		; soemthign else than ENTER or BS? then reget last char
gb_finish:
	; r6 holds upper nybble of result
	mov A,r6
	rl a
	rl a
	rl a
	rl a
	anl a,#0f0h		; mask upper nybble
	mov r6,a
	mov a,r7		; get lower nybble
	anl a,#00fh		; mask lower nybble
	orl a,r6		; OR upper nybble in and return result of byte read
	ret
	
	; validhex - checks if char in Acc is a valid hex digit
	; I: ACC - digit in ASCII to check
	; O: C = 0 if digit was valid hex, C=1 otherwise
	;    B - contains value of valid hex digit
validhex:
	clr	ACC.5			; make upper case
	cjne	a,#'0',vhtmp1		; If < '0', Return CY == 1
	jc vhl?p2
vhtmp1:
	cjne	a,#'9',vhtmp2		; If <= '9', Return CY == 0
	jc vhl?p1
vhtmp2:
	cjne	a,#'A',vhtmp3		; If < 'A', Return CY == 1
	jc vhl?p2
vhtmp3:
	cjne a,#'F',vhtmp4		; If > 'F', Return CY == 1
	sjmp vhtmp5
vhtmp4:	jnc vhl?p2
vhtmp5:

vhl?p1:	
	acall hexdigit2value		; convert digit to its value
	mov b,a
	clr	c			; CY == 0 Means Good Digit
	ret				; Return To Caller

vhl?p2: setb	c			; CY == 1 Means Bad Digit
	ret				; Return To Caller

hexdigit2value:
	anl A,#7fh			; auf 7 Bit verkleinern
	jb ACC.6,chn_Buchstabe		; Buchstabenzeichen?
	jb ACC.4,chn_Zahl		; Zahlzeichen?
	mov A,#0FFh			; Fehlerbyte zurckgeben
	ret				; .... zurck mit Fehler
chn_Buchstabe:
	anl A,#0Fh			; relevanten Teil maskieren
	add A,#09h			; Zehnerdekade mitz�len (-1, da 'A' 65 statt 64)
	ret				
chn_Zahl:
	anl A,#0Fh			; reinen Zahlenwert ausmaskieren
	ret				; und zurck...
		
	; dec_dptr: decrements current DPTR
	; I: none
	; O: DPTR=DPTR-1
dec_dptr:
    	xch     A,DPL               ;Exchange A for DPL
    	dec    A                   ;Decrement A (which is DPL)
    	cjne    A,#0FFh,_dec_dptr2  ;If A (DPL) is not #0FFh, continue normally
    	dec     DPH                 ;If A=FFh, we need to decrement DPH
_dec_dptr2:
    	xch     A,DPL               ;Exchange A for DPL (thus saving DPL and restoring A)
    	ret
	
	; register_dump - prints register list of the user space on the SIO
register_dump:
	mov dptr,#reg_list
	acall putstring
	mov A,userPCH
	acall putbyte
	mov A,userPCL
	acall putbyte
	PUT_SPACE
	mov A,userDPH
	acall putbyte
	mov A,userDPL
	acall putbyte
	PUT_SPACE2
	mov A,userPSW
	acall putbyte
	PUT_SPACE
	mov A,userSP
	acall putbyte
	PUT_SPACE
	mov A,userACC
	acall putbyte
	PUT_SPACE
	mov A,userB
	acall putbyte
	PUT_SPACE
	mov A,00h
	acall putbyte
	PUT_SPACE
	mov A,01h
	acall putbyte
	PUT_SPACE
	mov A,02h
	acall putbyte
	PUT_SPACE
	mov A,03h
	acall putbyte
	PUT_SPACE
	mov A,04h
	acall putbyte
	PUT_SPACE
	mov A,05h
	acall putbyte
	PUT_SPACE
	mov A,06h
	acall putbyte
	PUT_SPACE
	mov A,07h
	acall putbyte
	; and finish...
	CRLF
	ret
	
	; loadhex - routine to load Intel HEX files into XRAM
	; I: none
	; O: none (except for SIO messages)
loadhex:

	mov dptr,#loadhex_msg
	acall putstring
	
load1:  lcall getchar

	; stop loading on EOF, ESC or CR!!

	cjne A,#ESC,no_ESC_load ; ESC?
	sjmp done_load
no_ESC_load:
        cjne a,#1ah,no_EOF   ; end of file?
        sjmp done_load
no_EOF:
	cjne a,#CR,skip1 ; CR on line begin?
	sjmp done_load		

	; if no termination character was recieved wait for :
skip1:  cjne a,#':',load1   ; each record begins with ':'
        mov r1,#0           ; init checksum to zero
        lcall gethex        ; get byte from serial port
        mov b,a             ; use b as byte counter
        jz done_load             ; if b = 0

load2:  inc b
        lcall gethex        ; get address high byte
        mov dph,a
        lcall gethex        ; get address low byte
        mov dpl,a
        lcall gethex        ; get record type (ignore this)

load4:  acall gethex        ; get data byte
        movx @dptr,a        ; store in ext. ram
        inc dptr
        dec b               ; repeat until count = 0
        mov a,b
        jnz load4
        mov a,r1            ; checksum should be zero
        jz load1            ; if so, then get next record
                
	; LOAD ERROR!!!
	mov dptr,#loadhex_error
	acall putstring
	ret
done_load: 
	; LOAD was ok!
	mov dptr,#loadhex_ok
	acall putstring    
	ret
gethex: lcall getchar        ; get first character
        lcall atoh          ; convert to hex
        swap a              ; put in upper nibble
        mov r0,a            ; save it
        lcall getchar        ; get second character
        lcall atoh          ; convert to hex
        orl a,r0            ; or with first nibble
        mov r2,a            ; save byte
        add a,r1            ; add byte to checksum
        mov r1,a            ; restore checkum in r1
        mov a,r2            ; retrieve byte
        ret
atoh:	clr acc.7           ; ensure parity bit is off
        cjne a,#3ah,next    ; ?? this just sets carry bit ??
next:   jc atoh2
        add a,#9            ; no, adjust for range A-F
atoh2:  anl a,#0fh          ; yes, convert directly
        ret

	; userjmp - jump into the user program as specified by the user PC
	; DO NOT CALL THIS VECTOR - ALWAYS JUMP HERE
	; I: PC to be used in userPC
	; O: none
userjmp:
	; user program uses bank 0 for registers by default
	REGS_USER
	
	; initialize registers with the values for the user program
	mov A,userACC
	mov B,userB
	mov DPH,userDPH
	mov DPL,userDPL
	mov PSW,#0
	
	push userPCL	; push address of user code
	push userPCH
	ret		; and jump via stack reading to it

	; usercall - call the user program as specified by the user PC and warmstart afterwards
	; DO NOT CALL THIS VECTOR - ALWAYS JUMP HERE
	; I: PC to be used in userPC
	; O: none
usercall:
	; push return address (user program has to be terminated by RET)
	mov DPH,#HIGH(warmstart)
	mov DPL,#LOW(warmstart)
	push DPL
	push DPH
	
	; user program uses bank 0 for registers by default
	REGS_USER
	
	; initialize registers with the values for the user program
	mov A,userACC
	mov B,userB
	mov DPH,userDPH
	mov DPL,userDPL
	mov PSW,#0
	
	push userPCL	; push address of user code
	push userPCH
	ret		; and jump via stack reading to it
	
	; read_iram - displays the contents of a user given IRAM address
	; I: none
	; O: on SIO
	;
read_iram:
	mov dptr,#iram_msg		; prompt for IRAM address
	acall putstring			
	acall getbyte			; read address
	mov r0,A
read_iram2:	
	mov A,@r0			; fetch contents
	mov B,A				; store temporary
	CRLF
	mov dptr,#dump_iram_msg
	acall putstring
	mov A,r0
	acall putbyte
	mov A,#'='
	acall putchar
	mov A,B
	acall putbyte
	CRLF
	ret
	
	; store_iram - store a value in a user given IRAM address
	; I: none
	; O: on SIO
	;
store_iram:
	mov dptr,#iram_msg		; prompt for IRAM address
	acall putstring			
	acall getbyte			; read address
	mov r0,A	
	
	acall read_iram2
	
	mov dptr,#newvalue_msg
	acall putstring
	acall getbyte			; read new value
	mov @r0,A			; store it
	CRLF
	ret
	
	; dump_iram - dumps all IRAM locations in HEX
	; I: none
	; O: on SIO
dump_iram:
	mov r0,#00h			; start address of IRAM
dump_iram_lines:
	mov dptr,#dump_iram_msg
	acall putstring
	mov A,r0
	acall putbyte
	mov A,#'='
	acall putchar
	
	mov r4,#16			; loop 16 times
diram2:
	mov A,@r0			; display next IRAM loc
	acall putbyte
	mov A,#' '
	acall putchar
	inc r0				; advance IRAM loc
	djnz r4,diram2			; loop until we did dump the next 16 ones
	CRLF
	cjne r0,#0,dump_iram_lines	; stop dump when we did dump all 256 locations
	ret
	
	; read_sfr - infrastructure routine to read value of a SFR
	; I: ACC - address of SFR
	; O: ACC - value of SFR, 0xff if SFR address was <80h
	;
read_sfr:
	clr C
	subb A,#080h		; address in range? C=1 if A<80h
	jnc read_SFR_ok		; nope
	; SFR address invalid	
	mov A,#0ffh		; return dummy value
	ret
read_SFR_ok:
	clr ACC.7
	rl A			; A=(A-128)*2 => offset into read table
	mov dptr,#read_sfr_table
	jmp @A+DPTR		; do a vectored jump

	; store_sfr - infrastructure routine to store a value in a SFR
	; I: ACC - address of SFR
	;    r7 - value to be written into SFR
	; O: none
	;
store_sfr:
	clr C
	subb A,#080h		; address in range? C=1 if A<80h
	jnc write_SFR_ok	; nope
	; SFR address invalid	
	ret
write_SFR_ok:
	clr ACC.7
	rl A			; A=(A-128)*2 => offset into read table
	mov dptr,#write_sfr_table
	jmp @A+DPTR		; do a vectored jump
	
	; include jump table structure and routines for SFR reading
	; we have routs for reading and writing SFRs from 0x80 to 0xff
$INCLUDE(sfr_tables.inc)
	
	; dump_SFRs - dump a list of known SFRs to the SIO
	; I: none
	; O: none
	;
dump_SFRs:
	mov dptr,#sfr_dump_table
dump_sfr_loop:
	clr A
	movc A,@A+DPTR		; read SFR address
	jz stop_sfr_dump	; address = 0? then terminate dump
	
	push DPL
	push DPH
	lcall read_sfr		; read associated SFR	(alters DPTR!!!!!)
	pop DPH
	pop DPL
	
	mov r7,A		; store value temporary
	
	inc dptr		; point to info string
	lcall putstring		; r0 points to last 0
	; add 6 to DPTR => point to next record in table
	mov A,DPL
	add A,#6
	mov DPL,A
	mov A,DPH
	addc A,#0
	mov DPH,A
	
	push DPL
	push DPH
	mov A,r7		; get SFR value back and output
	lcall putbyte
	CRLF
	pop DPH
	pop DPL

	sjmp dump_sfr_loop
	
stop_sfr_dump:
	ret

	; poweron - at cold reset execution starts here...
	; I: none
	; O: none (HW initialized)
	;
poweron:
	mov	IP,#0
	mov	IE,#0			; all interrupts off
	
	; patch for PHYtec microMODUL-8051: monitor stack resides high up in IDATA space
        mov     SP,#stackloc
        
	; patch for PHYtec microMODUL-8051: enables von Neumann-Architecture for CSEG 0000 to 7fff
	mov	DPTR,#0FC02H
	mov	A,#10000000B
	movx	@DPTR,A

	inc	DPTR
	mov	A,#01111100B
	movx	@DPTR,A

	mov	DPTR,#0FC00H
	mov	A,#00010010B
	movx	@DPTR,A
	
	clr	ES
	
	; Tasking compatible reset of SFRs
	mov	P0,#0ffH
	mov	P1,#0ffH
	;	MOV	P3,#0ffH	Used by some boards for EPROM mapping! (e.g. Starter Kit 511/513)
	mov	SCON,#000H
	mov	TCON,#000H
	mov	TH0,#000H
	mov	TH1,#000H
	mov	TL0,#000H
	mov	TL1,#000H
	mov	TMOD,#000H
	
	REGS_MON
	
	mov	A,#52			; delay to finish serial transmitter
_mon51_delay:
	djnz	R2,_mon51_delay
	djnz	ACC,_mon51_delay
	        
	; patch for PHYtec microMODUL-8051: copy IRQ vectors to XRAM at 0000 
	; the vectors contain jumps into a program with interrupt table relocated to 0x100 and up
	; this allows to run SDCC output to code loc 0x100 directly without copying vectors
	;
	mov r0,#026h          ; bytes to copy 26h-1
	;mov r1,#026h          ; lower byte of destination address
	mov dptr,#monstart     ; DPTR points into my Flash
copyirqs:
	mov A,r0              	; load offset into my original table
	movc A,@A+dptr        	; get byte from code space
	
	mov P2,#HIGH(resetvector)		; set upper byte of reset vector to the XRAM bus
	movx @r0,A            	; lower byte on bus = offset - write the byte from CODE space there
	dec r0   	      	; decrement destination pointer
	djnz r0,copyirqs    	; loop for all bytes in charge
	
	; patch for PHYtec microMODUL-8051: init SIO
	mov 	SCON,#054h	; 8N1 - clr TI+RI
	mov	PCON,#010h
	
	mov	T2CON,#034h	; T2 as baudrate generator
	
	mov	T2RC2L,#LOW(baudrate)	; 9600 Baud at 12MHz
	mov	T2RC2H,#HIGH(baudrate)
	
	; preinit user registers
	
	mov userPCH,#HIGH(program)
	mov userPCL,#LOW(program)	; define user program PC
	mov userDPH,#0
	mov userDPL,#0
	mov userSP,#30h
	mov userACC,#0
	mov userB,#0
	mov userPSW,#0
	; preinit r0-r7 for user land
	mov r0,#07h
	mov A,#0ffh
clrregs:
	mov @r0,A
	djnz r0,clrregs
	mov 00h,#0ffh
	
	sjmp bypasswarmstart

	; warmstart - a reset and user ljmp 0x0000 will lead here and do a warmstart of the monitor
	; also a call of a user routine will return here
warmstart:
	mov IE,#0			; kill any active user interrupts
	mov IP,#0
	
	mov userDPH,DPH
	mov userDPL,DPL			; save user DPTR
	mov userSP,SP			; save user stack pointer
	mov userPSW,PSW			; save user PSW
	mov userACC,A			; save user ACC
	mov userB,B			; save user B

bypasswarmstart:
	setb TI
	clr RI
	REGS_MON
	; send WELCOME message
	mov dptr,#version
	lcall putstring	
	
	; main command loop	       
mainloop:
	; output the prompt
	mov A,#PROMPTCHAR
	lcall putchar			; send prompt
cmdloop:
	lcall getchar			; read command char
	; comment this out
	cjne A,#ESC,no_ESC		; ESC?
	; FIXME: remove for ROM version!
	ljmp 918eh			; Function: back to TASKING
no_ESC:
	cjne A,#CTRLC,no_CTRLC		; CTRL-C?
	mov dptr,#controlc
	lcall putstring			; say "Control+C" pressed
	ljmp monstart			; Function: restart monitor
no_CTRLC:
	cjne A,#CR,no_CR		; ENTER?	
	lcall register_dump 		; Function: do a user register dump
	ajmp mainloop
no_CR:
	cjne A,#'?',no_questmark	; "?"?
	lcall putchar			; Function: HELP screen
	mov dptr,#version		; Function: send version message
	lcall putstring	
	mov dptr,#helpscreen1
	lcall putstring
	mov dptr,#helpscreen2
	lcall putstring
	mov dptr,#helpscreen3
	lcall putstring
	mov dptr,#helpscreen4
	lcall putstring
	mov dptr,#helpscreen5
	lcall putstring
	mov dptr,#helpscreen6
	lcall putstring
	mov dptr,#helpscreen7
	lcall putstring
	mov dptr,#helpscreen8
	lcall putstring
	ajmp mainloop
no_questmark:	
	cjne A,#'v',no_v		; "v"?
	lcall putchar
	mov dptr,#version		; Function: send version message
	lcall putstring	
	ajmp mainloop
no_v:
	cjne A,#'A',no_a		; "A"?
	lcall putchar			; Function: write user ACC
	mov A,#'='			; prompt for value
	lcall putchar
	lcall getbyte	
	mov userACC,A			; store into user ACC
	CRLF				
	ajmp mainloop			; and prompt again
no_a:
	cjne A,#'B',no_b		; "B"?
	lcall putchar			; Function: write user B
	mov A,#'='			; prompt for value
	lcall putchar
	lcall getbyte	
	mov userB,A			; read user B
	CRLF				
	ajmp mainloop			; and prompt again
no_b:
	cjne A,#'P',no_bigP		; "P"?
	lcall putchar			; Function: Set user PC
	CRLF
	
	mov dptr,#notimpl
	lcall putstring
	
	ajmp mainloop
no_bigP:
	cjne A,#'D',no_bigD		; "D"?
	lcall putchar			; Function: Set user DPTR
	CRLF
	
	mov dptr,#notimpl
	lcall putstring
	
	ajmp mainloop
no_bigD:
	cjne A,#'j',no_j		; "j"?
	lcall putchar			; Function: jump to user programm via user PC
	CRLF
	ljmp userjmp			; no call or return - as we jump into the user code, return via reset vector
no_j:
	cjne A,#'C',no_bigc		; "C"?
	lcall putchar			; Function: call user programm via user PC
	CRLF
	ljmp usercall
no_bigc:
	cjne A,#'l',no_l		; "l"?
	lcall putchar			; Function: load Intel HEX file into memory
	CRLF
	lcall loadhex
	ljmp mainloop
no_l:
	cjne A,#'r',no_r		; "r"?
	lcall putchar			; Function: view IRAM
	CRLF
	lcall read_iram
	ljmp mainloop
no_r:
	cjne A,#'s',no_s		; "s"?
	lcall putchar			; Function: store in IRAM
	CRLF
	lcall store_iram
	ljmp mainloop
no_s:
	cjne A,#'R',no_bigR		; "R"?
	lcall putchar			; Function: view whole IRAM
	CRLF
	
	lcall dump_iram
	
	ljmp mainloop
no_bigR:
	cjne A,#'x',no_x		; "x"?
	lcall putchar			; Function: view XRAM
	CRLF
	
	mov dptr,#notimpl
	lcall putstring
	
	ajmp mainloop
no_x:
	cjne A,#'X',no_bigX		; "X"?
	lcall putchar			; Function: view whole XRAM
	CRLF
	
	mov dptr,#notimpl
	lcall putstring
	
	ajmp mainloop
no_bigX:
	cjne A,#'w',no_w		; "w"?
	lcall putchar			; Function: write XRAM
	CRLF
	
	mov dptr,#notimpl
	lcall putstring
	
	ajmp mainloop
no_w:
	cjne A,#'c',no_c		; "c"?
	lcall putchar			; Function: dump CODE memory
	CRLF
	
	mov dptr,#notimpl
	lcall putstring
	
	ajmp mainloop
no_c:
	cjne A,#'p',no_p		; "p"?
	lcall putchar			; Function: write SFR
	CRLF
	
	mov dptr,#notimpl
	lcall putstring
	
	ajmp mainloop
no_p:
	cjne A,#'S',no_bigS		; "S"?
	lcall putchar			; Function: view whole SFRs
	CRLF
	
	lcall dump_SFRs
	
	ajmp mainloop
no_bigS:
	; invalid command char so get another one....
	ajmp cmdloop
	
	; **** DATA segment ****
controlc:
	db "Control+C",13,10,13,10,0
notimpl:
	db "Function not implemented yet!",13,10,0
	
loadhex_msg:
	db "Upload Intel HEX file now - (ESC) to abort!",13,10,0
loadhex_error:
	db "Error in Intel HEX data!",13,10,0
loadhex_ok:
	db "Upload ok.",13,10,0

newvalue_msg:
	db "New value=",0	
iram_msg:
	db "IRAM loc=",0
dump_iram_msg:
	db "DBYTE ",0	
	
read_xram_msg:
	db "XRAM loc=",0
dump_xram_msg:
	db "XBYTE ",0
	
version:
	db 13,10,"ASM Software Monitor-52 v",VERSIONMAJOR+030h,'.',VERSIONMINOR+030h
crlfstr:
	db 13,10,0
reg_list:
	db 13,10,"PC   DPTR PSW SP A  B  R0 R1 R2 R3 R4 R5 R6 R7",13,10,0

	; HELP screen
helpscreen1:
	db 13,10,"HELP SCREEN",13,10,"-----------",13,10,0
helpscreen2:
	db "Type the key in brackets to activate the associated function.",13,10,"Input is case sensitive.",13,10,13,10,0
helpscreen3:
	;  "01234567890123456789"
	db "(P) set user PC     "
	db "(D) set user DPTR   "
	db "(A) enter user ACC  "
	db "(B) enter user B reg",13,10,0
helpscreen4:
	db "(l) load HEX file   "
	db "(j) jump to user PC "
	db "(C) call user PC",13,10,0
helpscreen5:
	db "(r) read onchip     "
	db "(s) store onchip    "
	db "(R) dump onchip RAM ",13,10,0
helpscreen6:
	db "(x) read ext RAM    "
	db "(w) write ext RAM   "
	db "(X) dump ext RAM    "
	db "(c) dump CODE memory",13,10,0
helpscreen7:
	db "(p) write SFR       "
	db "(S) dump SFR contents",13,10,13,10,0
helpscreen8:
	db "(ENTER)  show user registers",13,10
	db "(CTRL+C) cold reset monitor",13,10,0
	
sfr_dump_table:
	;      012345
	db P0,"  P0=",0
	db P1,"  P1=",0
	db P2,"  P2=",0
	db P3,"  P3=",0
	db PCON,"PCON=",0
	db TCON,"TCON=",0
	db TMOD,"TMOD=",0
	db TH0," TH0=",0
	db TL0," TL0=",0
	db TH1," TH1=",0
	db TL1," TL1=",0
	db SCON,"SCON=",0
	db IE,"  IE=",0
	db IP,"  IP=",0
	db 0,0,0
END
